@@ -11,10 +11,10 @@ import numpy as np
 import sys
 from pathlib import Path
 
-import flags
-import mwahpy_glob
-from timestep import Timestep
-from nbody import Nbody
+from .flags import verbose, progress_bars
+from .mwahpy_glob import file_len, progress_bar
+from .timestep import Timestep
+from .nbody import Nbody
 
 #===============================================================================
 # FUNCTIONS
@@ -24,7 +24,7 @@ from nbody import Nbody
 # INPUT
 #-------------------------------------------------------------------------------
 
-def removeHeader(f):
+def remove_header(f):
     #f (open file): the milkyway ".out" file
 
     comass = []
@@ -45,55 +45,99 @@ def removeHeader(f):
     return comass, comom
 
 #parses a milkyway ".out" file and returns a Timestep class structure
-def readOutput(f):
+def read_output(f):
     #f (str): the path of the milkyway ".out" file
 
-    if flags.progressBars:
-        flen = mwahpy_glob.fileLen(f)
-    if flags.verbose:
+    if progress_bars:
+        flen = file_len(f)
+    if verbose:
         print('\nReading in data from ' + str(f) + '...')
 
     f = open(f, 'r')
 
     #remove the header, get relevant info from header
-    comass, comom = removeHeader(f)
+    comass, comom = remove_header(f)
 
     #store the data here temporarily
-    #indexed this way to avoid the 'ignore' column
-    array_dict = {1:[], 2:[], 3:[], 4:[], 5:[], 6:[], 7:[], 8:[], 9:[], 10:[], 11:[]}
+    array_dict = {0:[], 1:[], 2:[], 3:[], 4:[], 5:[], 6:[], 7:[], 8:[], 9:[], 10:[], 11:[]}
 
     #place all the data from the file into the dictionary
-    if flags.progressBars:
+    if progress_bars:
         j = 0
     for line in f:
         line = line.strip().split(',')
-        i = 1
+        i = 0
         while i < len(line) - 1: #this grabs l, b, r data even though that is calculated from x, y, z in the Timestep class implementation
-                                 #  this is mostly just for simplicity when reading in, although I guess it probably slows down the code somewhat
+                                 #it's mostly just for simplicity when reading in, although I guess it probably slows down the code somewhat
             array_dict[i].append(float(line[i]))
             i += 1
-        if flags.progressBars:
+        if progress_bars:
             j += 1
-            mwahpy_glob.progressBar(j, flen)
+            progress_bar(j, flen)
 
     #return the Timestep class using the array dictionary we built
-    if flags.verbose:
+    if verbose:
         print('\n'+ str(len(array_dict[1])) + ' objects read in')
         sys.stdout.write('\rConverting data...')
-    d = Timestep(id_val=array_dict[1], x=array_dict[2], y=array_dict[3], z=array_dict[4], vx=array_dict[8], vy=array_dict[9], vz=array_dict[10], mass=array_dict[11], centerOfMass=comass, centerOfMomentum=comom)
-    if flags.verbose:
+    d = Timestep(typ=array_dict[0], id_val=array_dict[1], x=array_dict[2], y=array_dict[3], z=array_dict[4], vx=array_dict[8], vy=array_dict[9], vz=array_dict[10], mass=array_dict[11], center_of_mass=comass, center_of_momentum=comom)
+    if verbose:
         sys.stdout.write('done\n')
 
     f.close()
 
     return d
 
+#parses a milkyway ".in" file and returns a Timestep class structure
+def read_input(f):
+    #f (str): the path of the milkyway ".in" file
+
+    if progress_bars:
+        flen = file_len(f)
+    if verbose:
+        print('\nReading in data from ' + str(f) + '...')
+
+    f = open(f, 'r')
+
+    #remove the header
+    f.readline()
+
+    #store the data here temporarily
+    array_dict = {0:[], 1:[], 2:[], 3:[], 4:[], 5:[], 6:[], 7:[], 8:[]}
+
+    #place all the data from the file into the dictionary
+    if progress_bars:
+        j = 0
+    for line in f:
+        line = line.strip().split('\t')
+        i = 0
+        while i < len(line):
+            array_dict[i].append(float(line[i]))
+            i += 1
+        if progress_bars:
+            j += 1
+            progress_bar(j, flen)
+
+    #return the Timestep class using the array dictionary we built
+    if verbose:
+        print('\n'+ str(len(array_dict[1])) + ' objects read in')
+        sys.stdout.write('\rConverting data...')
+    d = Timestep(typ=array_dict[0], id_val=array_dict[1], x=array_dict[2], y=array_dict[3], z=array_dict[4], vx=array_dict[5], vy=array_dict[6], vz=array_dict[7], mass=array_dict[8], center_of_mass=[0,0,0], center_of_momentum=[0,0,0])
+    if verbose:
+        sys.stdout.write('done\n')
+
+    f.close()
+
+    d.update(force=True) #this generates Comass and Comomentum, as well as the other bits
+    #that you expect a Timestep to have initially when read in
+
+    return d
+
 #TODO: add nested progress bars
-def readFolder(f, ts_scale=None):
+def read_folder(f, ts_scale=None):
     #f: the path to the folder that you want to create an Nbody structure out of
     #ts_scale: the scale of a single timestep in the Nbody sim
 
-    if flags.verbose:
+    if verbose:
         print('\nReading in data from directory ' + str(f) + '...')
 
     n = Nbody(ts_scale=ts_scale) #create the Nbody instance
@@ -101,11 +145,10 @@ def readFolder(f, ts_scale=None):
     #iterate through the folder
     for i in Path(f).iterdir():
 
-        t = readOutput(str(i))
+        t = read_output(str(i))
         time = int(str(i).split('/')[-1])
         n[time] = t #assign each timestep to the correct place in the Nbody
 
-
     return n
 
 #-------------------------------------------------------------------------------
@@ -114,10 +157,10 @@ def readFolder(f, ts_scale=None):
 
 #parses a Timestep class object and outputs a file that can be read into a
 #MilkyWay@home N-body simulation as the 'manual bodies' parameter
-def makeNbodyInput(t, f):
+def make_nbody_input(t, f, recenter=True):
     #t (Timestep): the Timestep object that will be printed out
     #f (str): the path of the file that will be printed to
-    if flags.verbose:
+    if verbose:
         print('Writing Timestep as N-body input to '+f+'...')
 
     f = open(f, 'w')
@@ -125,23 +168,18 @@ def makeNbodyInput(t, f):
 
     i = 0
 
-    #recenter on center of mass for ease of inserting a dwarf
-    xnew = t.x - t.centerOfMass[0]
-    ynew = t.y - t.centerOfMass[1]
-    znew = t.z - t.centerOfMass[2]
-
-    while i < len(t):
-        f.write('\n1\t'+str(int(t.id[i]))+'\t'+str(xnew[i])+'\t'+str(ynew[i])+'\t'+str(znew[i])+'\t'+\
+    while i < len(t): #no idea why it's int(t.id[i]) instead of t.id[i]. Should change and see if it breaks.
+        f.write('\n'+str(t.typ[i])+'\t'+str(int(t.id[i]))+'\t'+str(xnew[i])+'\t'+str(ynew[i])+'\t'+str(znew[i])+'\t'+\
                 str(t.vx[i])+'\t'+str(t.vy[i])+'\t'+str(t.vz[i])+'\t'+str(t.mass[i]))
-        if flags.progressBars:
-            mwahpy_glob.progressBar(i, len(t))
+        if progress_bars:
+            progress_bar(i, len(t))
         i += 1
 
-    if flags.verbose:
+    if verbose:
         print('\ndone')
 
 #prints out a '.csv' file of the Timestep class structure
-def makeCSV(t, f_name):
+def make_csv(t, f_name):
     #t: the Timestep object being written out
     #f: the path for the output csv file
 
@@ -149,7 +187,7 @@ def makeCSV(t, f_name):
 
     #make the header
     #TODO: Print out COM's
-    if flags.verbose:
+    if verbose:
         print('Writing header...')
     header = ''
     for key in t:
@@ -159,13 +197,13 @@ def makeCSV(t, f_name):
 
     #iterate through the data and print each line
     i = 0
-    if flags.verbose:
+    if verbose:
         print('Printing data...')
-    while i < len(self): #i avoided zip() here because it's messy to zip
+    while i < len(t): #i avoided zip() here because it's messy to zip
     #                        like a billion different arrays, although zip()
     #                        is "better programming"
-        if flags.progressBars:
-            mwahpy_glob.progressBar(i, len(t))
+        if progress_bars:
+            progress_bar(i, len(t))
         line = ''
         for key in t:
             line += (str(t[key][i]) + ',')
